ad_page_contract {
    List predefined questions and allows editing

    @author  koljalehmann@uni.de
    @date    June 13, 2003
    @cvs-id $Id: predefined-question-ae-2.tcl,v 1.2.2.2 2004/01/13 20:30:50 maltes Exp $
} {
    predefined_question_id:optional
    question_text:html
    question_html_p:boolean
    presentation_type
}

set package_id [ad_conn package_id]
set user_id [ad_get_user_id]
set context_bar [ad_context_bar "Predefined Questions"]
# require sitewide admin
ad_require_permission [acs_magic_object "security_context_root"] "admin"

ad_form -name predef -form {
    predefined_question_id:key
    {question_text_new:text(hidden) {value $question_text}}
    {question_text:text(hidden) {value $question_text}}
    {question_html_p_new:text(hidden) {value $question_html_p}}
    {question_html_p:text(hidden) {value $question_html_p}}
    {presentation_type_new:text(hidden) {value $presentation_type}}
    {presentation_type:text(hidden) {value $presentation_type}}
    {question_text_inform:text(inform)    {label "Question"} {html {rows 5 cols 70}} {value $question_text}}

}

if {[info exist predefined_question_id]} {
    # get choices
    set valid_responses ""
    set choice_id_to_update_list [list]
    db_foreach survey_question_valid_responses "" {
	lappend choice_id_to_update_list $choice_id
	append valid_responses "$label\n"
    }
} else {
    set valid_responses ""
}


switch $presentation_type {
    textbox {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "One Line Answer"}}
	    {textbox_size:text(select) {options {{Small small} {Medium medium} {Large large}}} {label "Size"}}
	    {abstract_data_type:text(select) {label "Type of Response"}
                {options {{"Short Text" shorttext} {Text text} {Boolean boolean} {Number number} {Integer integer}}}
            }

}
    }
    textarea {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "Essay Answer"}}
	    {textarea_size:text(select) {options {{Small small} {Medium medium} {Large large}}} {label "Size"}}
	    {abstract_data_type:text(hidden) {value "text"}}
	}
    }
    "radio" -
    "select" {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "Multiple Choice"}}
	    {abstract_data_type:text(radio)
		{label "Type of Response"} {value "choice"} 
		{options {{"True or False" boolean} {"Yes or No" yn} {"Multiple Choice" choice}}}}
	    {valid_responses:text(textarea)
		{label "For Multiple Choice<br />Enter a List of Valid Responses<br /> (enter one choice per line)"}
		{html {rows 10 cols 50}} {value $valid_responses}}
	} -validate {
	    {valid_responses {$abstract_data_type!="choice" || ![empty_string_p $valid_responses]} "You did not enter a set of valid responses"}
	}

    }
    "checkbox" {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "Multiple Choice"}}
	    {valid_responses:text(textarea) {label "Valid Resposnes (enter one choice per line)"} {html {rows 10 cols 50}}}
	    {abstract_data_type:text(hidden) {value "choice"}}
	} -validate {
	    {valid_responses {$abstract_data_type!="choice" || ![empty_string_p $valid_responses]} "You did not enter a set of valid responses"}
	}
    }
    "date" {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "Date Answer"}}
	    {abstract_data_type:text(hidden) {value date}}
	}
    }
    "upload_file" {
	ad_form -extend -name predef -form {
	    {presentation:text(inform) {label "Presentation Type"} {value "Attachment Answer"}}
	    {abstract_data_type:text(hidden) {value blob}}
	}
    }
}

ad_form -extend -name predef -form {
#    {presentation_alignment:text(radio) {options {{"Beside the question" beside} {"Below the question" below}}} {value below} {label "Presentation Alignment"}}
    {presentation_alignment:text(hidden) {value beside}}
    {action_type:text(radio) {label "Action on submit"} {options {{"Update DB" db} {"Execute TCL-Code" tcl} {"Nothing" "null"}}}}
    {tcl:text(textarea),optional {label "TCL-Code"}}
    {table_name:text(text),optional {label "Table"}}
    {column_name:text(text),optional {label "Column"}}
    {key_name:text(text),optional {label "Key Column"}}
} -select_query {
    select * from survey_predefined_questions where predefined_question_id=:predefined_question_id
} -validate {
    {tcl {$action_type=="tcl" || [empty_string_p $tcl]} "Please select \"Execute TCL-Code\" if you want to enter code"}
    {tcl {![empty_string_p $tcl] || $action_type!="tcl"} "Please enter TCL-Code"}
    {table_name {$action_type=="db" || [empty_string_p $table_name]} "Please select \"Update DB\" if you want to insert values into the Database"}
    {table_name {![empty_string_p $table_name] || $action_type!="db"} "Please enter a table name"}
    {column_name {![empty_string_p $column_name] || $action_type!="db"} "Please enter a column name"}
    {column_name {$action_type=="db" || [empty_string_p $column_name]} "Please select \"Update DB\" if you want to insert values into the Database"}
    {key_name {$action_type=="db" || [empty_string_p $key_name]} "Please select \"Update DB\" if you want to insert values into the Database"}
    {key_name {![empty_string_p $key_name] || $action_type!="db"} "Please enter a key name"}
} -new_request {
    set action_type "null"
} -on_submit {
    if {$action_type=="null"} {
	set action_type [db_null]
    }
    # Generate presentation_options.
    set presentation_options ""
    if { $presentation_type == "textbox" } {
	if { [exists_and_not_null textbox_size] } {
	    # Will be "small", "medium", or "large".
	    set presentation_options $textbox_size
	}
    } elseif { $presentation_type == "textarea" } {
	if { [exists_and_not_null textarea_size] } {
	    # Will be "small", "medium", or "large".
	    set presentation_options $textarea_size
	}
    } elseif { $abstract_data_type == "yn" } {
	set abstract_data_type "boolean"
	set presentation_options "Yes/No"
    } elseif { $abstract_data_type == "boolean" } {
	set presentation_options "True/False"
    }    
} -new_data {
    db_transaction {
	db_exec_plsql new_predefined_question ""
	db_dml update_question_text ""
	if { $presentation_type == "checkbox" || $presentation_type == "radio" || $presentation_type == "select" } {
	    if { $abstract_data_type == "choice" } {
		set responses [split $valid_responses "\n"]
		set sort_order 0
		foreach response $responses {
		    set choice_label [string trim $response]
		    if { [empty_string_p $choice_label] } {
			# skip empty lines
			continue
		    } else {
		    ### added this next line to 
		    set new_choice_id [db_nextval survey_choice_id_sequence]
		    db_dml insert_new_choice ""
		    incr sort_order
		}
	        }
	    }
	}
    }
} -edit_data {    
    db_dml edit_predefined_question ""

	    set responses [split $valid_responses "\n"]
	    set count 0
	    set response_list ""
	    foreach response $responses {
		set trimmed_response [string trim $response]
		if { [empty_string_p $trimmed_response] } {
		    # skip empty lines
		    continue
		}
		
		lappend response_list [list "$trimmed_response" "$count"]
		incr count
	    }

	    set sort_order 0
	    foreach one_response $response_list {
		set choice_label [lindex $one_response 0]
		set sort_order [lindex $one_response 1]
		set choice_id_to_update [lindex $choice_id_to_update_list $sort_order]
		if {[empty_string_p $choice_id_to_update]} {
		    set new_choice_id [db_nextval survey_choice_id_sequence]
		    db_dml insert_new_choice {}
		} else {
		    
		    db_dml update_new_choice {}
		}
		incr sort_order
	    }
	    while {[llength $choice_id_to_update_list] >= $sort_order} {
		set choice_id_to_delete [lindex $choice_id_to_update_list $sort_order]
		db_dml delete_old_choice {}
		incr sort_order
	    }
	    
} -after_submit {
    ad_returnredirect predefined-question-admin
    ad_script_abort
}

ad_return_template



